import sys
import asyncio

if sys.version_info[0] == 3 and sys.version_info[1] >= 8 and sys.platform.startswith('win'):
    asyncio.set_event_loop_policy(
        asyncio.WindowsSelectorEventLoopPolicy())



import unittest

import apraw


class CommentTest(unittest.IsolatedAsyncioTestCase):
    def __init__(self, *args, **kwargs):
        super(CommentTest, self).__init__(*args, **kwargs)

        self._reddit = apraw.Reddit("APB")

    async def asyncSetUp(self):
        self._comment = await self._reddit.comment("fulsybg")

    async def test_comment_author(self):
        author = await self._comment.author()
        self.assertEqual(author.name, "Dan6erbond")

    async def test_comment_submission(self):
        submission = await self._comment.submission()
        self.assertEqual(submission.id, "h7mna9")


if __name__ == "__main__":
    unittest.main()



import unittest

import apraw


class ListingGeneratorTest(unittest.IsolatedAsyncioTestCase):
    def __init__(self, *args, **kwargs):
        super(ListingGeneratorTest, self).__init__(*args, **kwargs)

        self._reddit = apraw.Reddit("APB")

    async def asyncSetUp(self):
        subreddit = await self._reddit.subreddit("aprawtest")
        self._listing_generator = subreddit.new

    async def test_listing_generator_get(self):
        submission_found = False

        async for submission in self._listing_generator.get():
            if submission.id == "h7mna9":
                submission_found = True
                break

        self.assertTrue(submission_found)

    async def test_listing_generator_call(self):
        self.assertEqual(self._listing_generator.__call__, self._listing_generator.get)

    async def test_listing_generator_stream(self):
        submission_found = False

        async for submission in self._listing_generator.stream():
            if submission.id == "h7mna9":
                submission_found = True
                break

        self.assertTrue(submission_found)



if __name__ == "__main__":
    unittest.main()



import unittest

import apraw


class RedditorTest(unittest.IsolatedAsyncioTestCase):
    def __init__(self, *args, **kwargs):
        super(RedditorTest, self).__init__(*args, **kwargs)

        self._reddit = apraw.Reddit("APB")

    async def asyncSetUp(self):
        self._redditor = await self._reddit.redditor("aprawbot")

    async def test_redditor_moderated_subreddits(self):
        subreddit_found = False

        async for subreddit in self._redditor.moderated_subreddits():
            if subreddit.display_name.lower() == "aprawtest":
                subreddit_found = True
                break

        self.assertTrue(subreddit_found)

    async def test_redditor_comments(self):
        comment_found = False

        async for comment in self._redditor.comments(limit=None):
            if comment.id == "fuoew5r":
                comment_found = True
                break

        self.assertTrue(comment_found)

    async def test_redditor_submissions(self):
        submission_found = False

        async for submission in self._redditor.submissions(limit=None):
            if submission.id == "h81irf":
                submission_found = True
                break

        self.assertTrue(submission_found)


if __name__ == "__main__":
    unittest.main()



import unittest

import apraw


class SubmissionTest(unittest.IsolatedAsyncioTestCase):
    def __init__(self, *args, **kwargs):
        super(SubmissionTest, self).__init__(*args, **kwargs)

        self._reddit = apraw.Reddit("APB")

    async def asyncSetUp(self):
        self._submission = await self._reddit.submission("h7mna9")

    async def test_submission_full_data(self):
        full_data = await self._submission.full_data()
        self.assertEqual(
            full_data[0]["data"]["children"][0]["data"]["id"],
            "h7mna9")

    async def test_submission_comments(self):
        comment_found = False

        async for comment in self._submission.comments():
            if comment.id == "fulsybg":
                comment_found = True
                break

        self.assertTrue(comment_found)

    async def test_submission_morechildren(self):
        children = ["fulsybg"]

        comment_found = False

        for comment in await self._submission.morechildren(children):
            if comment.id == "fulsybg":
                comment_found = True
                break

        self.assertTrue(comment_found)

    async def test_submission_subreddit(self):
        subreddit = await self._submission.subreddit()
        self.assertTrue(subreddit.display_name.lower() == "aprawtest")

    async def test_submission_author(self):
        author = await self._submission.author()
        self.assertTrue(author.name.lower() == "dan6erbond")


if __name__ == "__main__":
    unittest.main()



from ..endpoints import API_PATH


class SubredditModmail:
    def __init__(self, subreddit):
        self.subreddit = subreddit

    async def conversations(self):
        req = await self.subreddit.reddit.get_request(API_PATH["modmail_conversations"], entity=self.subreddit.display_name)
        for id in req["conversations"]:
            yield ModmailConversation(self.subreddit.reddit, req["conversations"][id])


class ModmailConversation:
    def __init__(self, reddit, data, owner=None):
        self.reddit = reddit
        self.data = data
        self._data = None

        self.id = data["id"]
        self._owner = owner

        self.subject = data["subject"]

        self.is_auto = data["isAuto"]
        self.obj_ids = data["objIds"]
        self.is_repliable = data["isRepliable"]
        self.last_user_update = data["lastUserUpdate"]
        self.is_internal = data["isInternal"]
        self.lastModUpdate = data["lastModUpdate"]
        self.lastUpdated = data["lastUpdated"]
        self.is_highlighted = data["isHighlighted"]
        self.state = data["state"]
        self.last_unread = data["lastUnread"]
        self.num_messages = data["numMessages"]

        self.authors = data["authors"]
        self.participant = data["participant"]

    async def owner(self):
        if self._owner is None:
            self._owner = await self.reddit.subreddit(self.data["owner"]["displayName"])
        return self._owner

    async def messages(self):
        full_data = await self.full_data()
        for msg_id in full_data["messages"]:
            yield ModmailMessage(self, full_data["messages"][msg_id])

    async def full_data(self):
        if self._data is None:
            self._data = await self.reddit.get_request(API_PATH["modmail_conversation"].format(id=self.id))
        return self._data

class ModmailMessage:
    def __init__(self, conversation, data):
        self.conversation = conversation
        self.data = data

        self.id = data["id"]

        self.body = data["body"]
        self.body_md = data["bodyMarkdown"]
        self._author = None
        self.is_internal = data["isInternal"]
        self.date = data["date"]

    async def author(self):
        if self._author is None:
            if not self.data["author"]["isDeleted"]:
                self._author = self.conversation.reddit.redditor(self.data["author"]["name"])
            else:
                return None
        return self._author



from datetime import datetime

from ..endpoints import API_PATH


class Comment:

    def __init__(self, reddit, data, submission=None, author=None):
        self.reddit = reddit
        self.data = data

        self._submission = submission
        self._author = author

        self.id = data["id"]
        self.created_utc = datetime.utcfromtimestamp(data["created_utc"])

        self.edited = data["edited"]
        self.archived = data["archived"]
        self.link_id = data["link_id"]
        self.parent_id = data["parent_id"]
        self.score = data["score"]
        self.body = data["body"]
        self.is_submitter = data["is_submitter"]
        self.url = "https://www.reddit.com" + data["permalink"]

        self.user_reports = data["user_reports"]
        self.mod_reports = data["mod_reports"]

    async def author(self):
        if self._author is None:
            self._author = await self.reddit.redditor(self.data["author"])
        return self._author

    async def submission(self):
        if self._submission is None:
            link = await self.reddit.get_request(API_PATH["info"], id=self.data["link_id"])
            from .submission import Submission
            self._submission = Submission(
                self.reddit, link["data"]["children"][0]["data"])
        return self._submission



import asyncio
import configparser
import logging
from datetime import datetime, timedelta

import aiohttp

from .endpoints import API_PATH, BASE_URL
from .models import Comment, Redditor, Submission, Subreddit


class Reddit:

    def __init__(self, praw_key="", username="", password="", client_id="", client_secret="",
                 user_agent="aPRAW by Dan6erbond"):
        if praw_key != "":
            config = configparser.ConfigParser()
            config.read("praw.ini")

            self.auth = Auth(
                config[praw_key]["username"],
                config[praw_key]["password"],
                config[praw_key]["client_id"],
                config[praw_key]["client_secret"],
                config[praw_key]["user_agent"] if "user_agent" in config[praw_key] else user_agent)
        else:
            self.auth = Auth(
                username,
                password,
                client_id,
                client_secret,
                user_agent)

        self.comment_kind = "t1"
        self.account_kind = "t2"
        self.link_kind = "t3"
        self.message_kind = "t4"
        self.subreddit_kind = "t5"
        self.award_kind = "t6"
        self.modaction_kind = "modaction"

        self.subreddits = Subreddits(self)
        self.request_handler = RequestHandler(self.auth)

    async def get_request(self, endpoint="", **kwargs):
        return await self.request_handler.get_request(endpoint, **kwargs)

    async def get_listing(self, endpoint, limit, **kwargs):
        last = None
        while True:
            kwargs["limit"] = limit if limit is not None else 100
            if last is not None:
                kwargs["after"] = last
            req = await self.get_request(endpoint, **kwargs)
            if len(req["data"]["children"]) <= 0:
                break
            for i in req["data"]["children"]:
                if i["kind"] in [self.link_kind,
                                 self.subreddit_kind, self.comment_kind]:
                    last = i["data"]["name"]
                elif i["kind"] == self.modaction_kind:
                    last = i["data"]["id"]

                if limit is not None:
                    limit -= 1
                yield i
            if limit is not None and limit < 1:
                break

    async def post_request(self, endpoint="", url="", data={}, **kwargs):
        return await self.request_handler.post_request(endpoint, url, data, **kwargs)

    async def subreddit(self, display_name):
        resp = await self.get_request(API_PATH["subreddit_about"].format(sub=display_name))
        try:
            return Subreddit(self, resp["data"])
        except Exception as e:
            logging.error(e)
            return None

    async def info(self, id="", url=""):
        # TODO: implement
        pass

    async def submission(self, id="", url=""):
        if id != "":
            id = self.link_kind + "_" + id.replace(self.link_kind + "_", "")
            link = await self.get_request(API_PATH["info"], id=id)
            if link["data"]["children"][0]["kind"] == self.link_kind:
                return Submission(self, link["data"]["children"][0]["data"])
        elif url != "":
            link = await self.get_request(API_PATH["info"], url=url)
            if link["data"]["children"][0]["kind"] == self.link_kind:
                return Submission(self, link["data"]["children"][0]["data"])
        return None

    async def comment(self, id="", url=""):
        if id != "":
            id = self.comment_kind + "_" + \
                id.replace(self.comment_kind + "_", "")
            comment = await self.get_request(API_PATH["info"], id=id)
            if comment["data"]["children"][0]["kind"] == self.comment_kind:
                return Comment(self, comment["data"]["children"][0]["data"])
        elif url != "":
            comment = await self.get_request(API_PATH["info"], url=url)
            if comment["data"]["children"][0]["kind"] == self.comment_kind:
                return Comment(self, comment["data"]["children"][0]["data"])
        return None

    async def redditor(self, username):
        resp = await self.get_request(API_PATH["user_about"].format(user=username))
        try:
            return Redditor(self, resp["data"])
        except Exception as e:
            # print("No Redditor data loaded for {}.".format(username))
            return None

    async def message(self, to, subject, text, from_sr=""):
        data = {
            "subject": subject,
            "text": text,
            "to": to
        }
        if from_sr != "":
            data["from_sr"] = from_sr
        resp = await self.post_request(API_PATH["compose"], data=data)
        return resp["success"]


class Subreddits:

    def __init__(self, reddit):
        self.reddit = reddit

    async def new(self, limit=25, **kwargs):
        async for s in self.reddit.get_listing(API_PATH["subreddits_new"], limit, **kwargs):
            if s["kind"] == self.reddit.subreddit_kind:
                yield Subreddit(self.reddit, s["data"])


class Auth:

    def __init__(self, username, password, client_id,
                 client_secret, user_agent):
        self.username = username
        self.password = password
        self.client_id = client_id
        self.client_secret = client_secret
        self.user_agent = user_agent

        if self.username == "" or self.password == "" or self.client_id == "" or self.client_secret == "":
            raise Exception(
                "No login information given or login information incomplete.")

        self.access_data = None
        self.token_expires = datetime.now()

        self.ratelimit_remaining = 0
        self.ratelimit_used = 0
        self.ratelimit_reset = datetime.now()


class RequestHandler:

    def __init__(self, auth):
        self.auth = auth
        self.queue = []

    async def get_request_headers(self):
        if self.auth.token_expires <= datetime.now():
            url = "https://www.reddit.com/api/v1/access_token"
            data = {
                "grant_type": "password",
                "username": self.auth.username,
                "password": self.auth.password
            }

            auth = aiohttp.BasicAuth(
                login=self.auth.client_id,
                password=self.auth.client_secret)
            async with aiohttp.ClientSession(auth=auth) as session:
                async with session.post(url, data=data) as resp:
                    if resp.status == 200:
                        self.auth.access_data = await resp.json()
                        self.auth.token_expires = datetime.now()
                        + timedelta(seconds=self.auth.access_data["expires_in"])
                    else:
                        raise Exception("Invalid user data.")

        return {
            "Authorization": "{} {}".format(self.auth.access_data["token_type"], self.auth.access_data["access_token"]),
            "User-Agent": self.auth.user_agent
        }

    def update(self, data):
        self.auth.ratelimit_remaining = int(
            float(data["x-ratelimit-remaining"]))
        self.auth.ratelimit_used = int(data["x-ratelimit-used"])

        self.auth.ratelimit_reset = datetime.now()
        + timedelta(seconds=int(data["x-ratelimit-reset"]))

    def check_ratelimit(func):
        async def execute_request(self, *args, **kwargs):
            id = datetime.now().strftime('%Y%m%d%H%M%S')
            self.queue.append(id)

            if (self.auth.ratelimit_remaining < 5):
                execution_time = self.auth.ratelimit_reset
                + timedelta(seconds=len(self.queue))
                wait_time = (execution_time - datetime.now()).total_seconds()
                asyncio.sleep(wait_time)

            result = await func(self, *args, **kwargs)
            self.queue.remove(id)
            return result

        return execute_request

    @check_ratelimit
    async def get_request(self, endpoint="", **kwargs):
        kwargs["raw_json"] = 1
        params = ["{}={}".format(k, kwargs[k]) for k in kwargs]

        url = BASE_URL.format(endpoint, "&".join(params))

        async with aiohttp.ClientSession() as session:
            headers = await self.get_request_headers()
            async with session.get(url, headers=headers) as resp:
                self.update(resp.headers)
                return await resp.json()

    @check_ratelimit
    async def post_request(self, endpoint="", url="", data={}, **kwargs):
        kwargs["raw_json"] = 1
        params = ["{}={}".format(k, kwargs[k]) for k in kwargs]

        if endpoint != "":
            url = BASE_URL.format(endpoint, "&".join(params))
        elif url != "":
            url = "{}?{}".format(url, "&".join(params))

        async with aiohttp.ClientSession() as session:
            headers = await self.get_request_headers()
            async with session.post(url, data=data, headers=headers) as resp:
                self.update(resp.headers)
                return await resp.json()



from datetime import datetime

from ..endpoints import API_PATH
from .comment import Comment


class Submission:

    def __init__(self, reddit, data, full_data=None, subreddit=None, author=None):
        self.reddit = reddit
        self.data = data

        self._full_data = full_data
        self._comments = list()
        self._subreddit = subreddit
        self._author = author

        self.id = data["id"]
        self.created_utc = datetime.utcfromtimestamp(data["created_utc"])

        self.name = data["name"]
        self.title = data["title"]
        self.selftext = data["selftext"]
        self.media = data["media"]
        self.over18 = data["over_18"]
        self.is_video = data["is_video"]
        self.original_content = data["is_original_content"]
        self.score = data["score"]
        self.stickied = data["stickied"]
        self.archived = data["archived"]
        self.locked = data["locked"]
        self.permalink = data["permalink"]
        self.url = data["url"]

        self.pinned = data["pinned"] # indicates if the post is pinned on the user's profile

        self.user_reports = data["user_reports"]
        self.mod_reports = data["mod_reports"]

    async def full_data(self):
        if self._full_data is None:
            sub = await self.subreddit()
            self._full_data = await self.reddit.get_request(API_PATH["submission"].format(sub=sub.display_name, id=self.id))
        return self._full_data

    async def comments(self, reload=False, **kwargs):
        if len(self._comments) <= 0 or reload:
            fd = await self.full_data()
            self._comments = list()

            for c in fd[1]["data"]["children"]:
                if c["kind"] == self.reddit.comment_kind:
                    self._comments.append(Comment(self.reddit, c["data"], submission=self))
                if c["kind"] == "more":
                    self._comments.extend(await self.morechildren(c["data"]["children"]))
        for c in self._comments:
            yield c

    async def morechildren(self, children):
        comments = list()

        while len(children) > 0:
            cs = children[:100]
            children = children[100:]

            data = await self.reddit.get_request(API_PATH["morechildren"], children=",".join(cs), link_id=self.name)
            for l in data["jquery"]:
                for _l in l:
                    if isinstance(_l, list):
                        for cl in _l:
                            if isinstance(cl, list):
                                for c in cl:
                                    if isinstance(c, dict) and "kind" in c and c["kind"] == self.reddit.comment_kind:
                                        comments.append(Comment(self.reddit, c["data"], submission=self))

        return comments

    async def subreddit(self):
        if self._subreddit is None:
            self._subreddit = await self.reddit.subreddit(self.data["subreddit"])
        return self._subreddit

    async def author(self):
        if self._author is None:
            self._author = await self.reddit.redditor(self.data["author"])
        return self._author



import unittest

import apraw


class RedditTest(unittest.IsolatedAsyncioTestCase):
    def __init__(self, *args, **kwargs):
        super(RedditTest, self).__init__(*args, **kwargs)

        self._reddit = apraw.Reddit("APB")

    async def test_reddit_subreddit(self):
        subreddit = await self._reddit.subreddit("aPRAWTest")
        self.assertEqual(subreddit.description, "Testing subreddit for aPRAW.")

    async def test_reddit_submission(self):
        submission = await self._reddit.submission("h7mna9")
        self.assertEqual(submission.title, "Test Post")

    async def test_reddit_comment(self):
        comment = await self._reddit.comment("fulsybg")
        self.assertEqual(comment.body, "This is a test comment.")

    async def test_reddit_redditor(self):
        redditor = await self._reddit.redditor("Dan6erbond")
        self.assertEqual(redditor.id, "11qzch")


if __name__ == "__main__":
    unittest.main()



import unittest

import apraw


class ModmailTest(unittest.IsolatedAsyncioTestCase):
    def __init__(self, *args, **kwargs):
        super(ModmailTest, self).__init__(*args, **kwargs)

        self._reddit = apraw.Reddit("APB")

    async def asyncSetUp(self):
        subreddit = await self._reddit.subreddit("aprawtest")
        self._modmail = subreddit.modmail

    async def test_modmail_conversations(self):
        conversation = None

        async for conv in self._modmail.conversations():
            if conv.subject == "invitation to moderate /r/aPRAWTest":
                conversation = conv
                break

        self.assertNotEqual(conversation, None)
        self.assertEqual(conversation.id, "er3yc")


if __name__ == "__main__":
    unittest.main()



import asyncio
from datetime import datetime

from ..endpoints import API_PATH
from .comment import Comment
from .modmail import SubredditModmail
from .submission import Submission


class Subreddit:
    def __init__(self, reddit, data):
        self.reddit = reddit
        self.data = data
        self.mod = SubredditModeration(self)
        self.modmail = SubredditModmail(self)

        self.id = data["id"]
        self.created_utc = datetime.utcfromtimestamp(data["created_utc"])

        self.display_name = data["display_name"]
        self.public_description = data["public_description"]
        self.description = data["description"]
        self.subscribers = data["subscribers"]
        self.quarantine = data["quarantine"] if "quarantine" in data else False
        self.subreddit_type = data["subreddit_type"]
        self.over18 = data["over18"] if "over18" in data else data["over_18"] if "over_18" in data else False
        self.user_is_subscribed = data["user_is_subscriber"]
        self.user_is_moderator = data["user_is_moderator"]

        from .listing_generator import ListingGenerator
        self.comments = ListingGenerator(
            self.reddit, API_PATH["subreddit_comments"].format(
                sub=self.display_name))
        self.new = ListingGenerator(self.reddit,
                                    API_PATH["subreddit_new"].format(sub=self.display_name))
        self.hot = ListingGenerator(self.reddit,
                                    API_PATH["subreddit_hot"].format(sub=self.display_name))
        self.rising = ListingGenerator(
            self.reddit, API_PATH["subreddit_rising"].format(sub=self.display_name))
        self.top = ListingGenerator(self.reddit,
                                    API_PATH["subreddit_top"].format(sub=self.display_name))

    def __str__(self):
        return self.display_name

    async def moderators(self, **kwargs):
        req = await self.reddit.get_request(API_PATH["subreddit_moderators"].format(sub=self.display_name), **kwargs)
        for u in req["data"]["children"]:
            yield SubredditModerator(self.reddit, u)

    async def message(self, subject, text, from_sr=""):
        return await self.reddit.message(API_PATH["subreddit"].format(sub=self.display_name), subject, text, from_sr)


class SubredditStream():
    def __init__(self, subreddit):
        self.subreddit = subreddit

    async def comments(self):
        # TODO: implement
        pass

    async def submissions(self, max_wait=16, **kwargs):
        wait = 0
        ids = list()

        while True:
            found = False
            async for s in self.subreddit.new(100, **kwargs):
                if s.id in ids:
                    break
                if len(ids) >= 301:
                    ids = ids[1:]
                ids.append(s.id)
                found = True
                yield s

            if found:
                wait = 1
            else:
                wait *= 2
                if wait > max_wait:
                    wait = 1

            print(wait)
            await asyncio.sleep(wait)


class SubredditModerator():
    def __init__(self, reddit, data):
        self.reddit = reddit
        self.data = data

        self.id = data["id"]

        self.name = data["name"]
        self.author_flair_text = data["author_flair_text"]
        self.author_flair_css_class = data["author_flair_css_class"]
        self.mod_permissions = data["mod_permissions"]
        self.added = data["date"]

    def __str__(self):
        return self.name

    async def redditor(self):
        return await self.reddit.redditor(self.name)


class SubredditModeration:
    def __init__(self, subreddit):
        self.subreddit = subreddit

    async def reports(self, limit=25, **kwargs):
        async for s in self.subreddit.reddit.get_listing(API_PATH["subreddit_reports"].format(sub=self.subreddit.display_name),
                                                         limit, **kwargs):
            if s["kind"] == self.subreddit.reddit.link_kind:
                yield Submission(self.subreddit.reddit, s["data"], subreddit=self.subreddit)
            elif s["kind"] == self.subreddit.reddit.comment_kind:
                yield Comment(self.subreddit.reddit, s["data"])

    async def spam(self, limit=25, **kwargs):
        async for s in self.subreddit.reddit.get_listing(API_PATH["subreddit_spam"].format(sub=self.subreddit.display_name),
                                                         limit, **kwargs):
            if s["kind"] == self.subreddit.reddit.link_kind:
                yield Submission(self.subreddit.reddit, s["data"], subreddit=self.subreddit)
            elif s["kind"] == self.subreddit.reddit.comment_kind:
                yield Comment(self.subreddit.reddit, s["data"])

    async def modqueue(self, limit=25, **kwargs):
        async for s in self.subreddit.reddit.get_listing(API_PATH["subreddit_modqueue"].format(sub=self.subreddit.display_name),
                                                         limit, **kwargs):
            if s["kind"] == self.subreddit.reddit.link_kind:
                yield Submission(self.subreddit.reddit, s["data"], subreddit=self.subreddit)
            elif s["kind"] == self.subreddit.reddit.comment_kind:
                yield Comment(self.subreddit.reddit, s["data"])

    async def unmoderated(self, limit=25, **kwargs):
        async for s in self.subreddit.reddit.get_listing(API_PATH["subreddit_unmoderated"].format(sub=self.subreddit.display_name),
                                                         limit, **kwargs):
            if s["kind"] == self.subreddit.reddit.link_kind:
                yield Submission(self.subreddit.reddit, s["data"], subreddit=self.subreddit)
            elif s["kind"] == self.subreddit.reddit.comment_kind:
                yield Comment(self.subreddit.reddit, s["data"])

    async def edited(self, limit=25, **kwargs):
        async for s in self.subreddit.reddit.get_listing(API_PATH["subreddit_edited"].format(sub=self.subreddit.display_name),
                                                         limit, **kwargs):
            if s["kind"] == self.subreddit.reddit.link_kind:
                yield Submission(self.subreddit.reddit, s["data"], subreddit=self.subreddit)
            elif s["kind"] == self.subreddit.reddit.comment_kind:
                yield Comment(self.subreddit.reddit, s["data"])

    async def log(self, limit=25, **kwargs):
        async for l in self.subreddit.reddit.get_listing(API_PATH["subreddit_log"].format(sub=self.subreddit.display_name),
                                                         limit, **kwargs):
            yield ModAction(l["data"], self.subreddit)


class ModAction:
    def __init__(self, data, subreddit=None):
        self.data = data
        self.subreddit = subreddit

        self.id = data["id"]

        self.created_utc = datetime.utcfromtimestamp(data["created_utc"])

        self.mod = data["mod"]
        self.description = data["description"]
        self.details = data["details"]
        self.action = data["action"]

        self.target_body = data["target_body"]
        self.target_title = data["target_title"]
        self.target_permalink = data["target_permalink"]
        self.target_author = data["target_author"]
        self.target_fullname = data["target_fullname"]

    async def mod(self):
        return await self.subreddit.reddit.redditor(self.mod)



"""List of Reddit API endpoints known to aPRAW."""


BASE_URL = "https://oauth.reddit.com{}?{}"

API_PATH = {
    "compose"              : "/api/compose",
    "info"                 : "/api/info",
    "moderated"            : "/user/{user}/moderated_subreddits",
    "modmail_conversation" : "/api/mod/conversations/{id}",
    "modmail_conversations": "/api/mod/conversations",
    "morechildren"         : "/api/morechildren",
    "submission"           : "/r/{sub}/comments/{id}",
    "subreddit"            : "/r/{sub}",
    "subreddit_about"      : "/r/{sub}/about",
    "subreddit_comments"   : "/r/{sub}/comments",
    "subreddit_edited"     : "/r/{sub}/about/edited",
    "subreddit_hot"        : "/r/{sub}/hot",
    "subreddit_log"        : "/r/{sub}/about/log",
    "subreddit_moderators" : "/r/{sub}/about/moderators",
    "subreddit_modqueue"   : "/r/{sub}/about/modqueue",
    "subreddit_new"        : "/r/{sub}/new",
    "subreddit_reports"    : "/r/{sub}/about/reports",
    "subreddit_rising"     : "/r/{sub}/rising",
    "subreddit_spam"       : "/r/{sub}/about/spam",
    "subreddit_top"        : "/r/{sub}/top",
    "subreddit_unmoderated": "/r/{sub}/about/unmoderated",
    "subreddits_new"       : "/subreddits/new",
    "user_about"           : "/user/{user}/about",
    "user_comments"        : "/user/{user}/comments",
    "user_submissions"     : "/user/{user}/submitted"
}



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="aPRAW",
    version="0.1.1-alpha",
    author="Dan6erbond",
    author_email="moravrav@gmail.com",
    description="Asynchronous Python Reddit API Wrapper",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Dan6erbond/aPRAW",
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Topic :: Education",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: Message Boards",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Typing :: Typed"
    ],
    python_requires='>=3.6',
)

# classifiers can be found here: https://pypi.org/classifiers/



from datetime import datetime

from ..endpoints import API_PATH
from .comment import Comment
from .submission import Submission
from .subreddit import Subreddit


class Redditor:
    def __init__(self, reddit, data):
        self.reddit = reddit
        self.data = data

        self.name = data["name"]

        if "is_suspended" not in data or not data["is_suspended"]:
            self.id = data["id"]
            self.is_suspended = False
            self.created_utc = datetime.utcfromtimestamp(data["created_utc"])

            self.is_employee = data["is_employee"]
            # self.has_visited_new_profile = data["has_visited_new_profile"]
            self.is_friend = data["is_friend"]
            # self.pref_no_profanity = data["pref_no_profanity"]
            # self.has_external_account = data["has_external_account"]
            # self.is_sponsor = data["is_sponsor"]
            # self.has_gold_subscription = data["has_gold_subscription"]
            # self.num_friends = data["num_friends"]
            self.verified = data["verified"]
            # self.over18 = data["over_18"] if "over18" in data else data["subreddit"]["over_18"] if "over_18" in data["subreddit"] else False
            self.is_gold = data["is_gold"]
            self.is_mod = data["is_mod"]
            self.has_verified_email = data["has_verified_email"]
            # self.pref_video_autoplay = data["pref_video_autoplay"]
            # self.in_chat = data["in_chat"]
            # self.in_redesign_beta = data["in_redesign_beta"]
            # self.pref_nightmode = data["pref_nightmode"]

            self.link_karma = data["link_karma"]
            self.comment_karma = data["comment_karma"]
        else:
            self.is_suspended = True

        if "subreddit" in data and data["subreddit"] is not None:
            sub = data["subreddit"]
            sub["id"] = sub["name"].replace("t5_", "")
            if "created_utc" not in sub:
                sub["created_utc"] = data["created_utc"]
            # sub["over18"] = self.over18
            self.subreddit = Subreddit(self.reddit, sub)
        else:
            self.subreddit = None

    def __str__(self):
        return self.name

    async def moderated_subreddits(self, **kwargs):
        req = await self.reddit.get_request(API_PATH["moderated"].format(user=self), **kwargs)
        for s in req["data"]:
            yield await self.reddit.subreddit(s["sr"])

    async def message(self, subject, text, from_sr=""):
        return await self.reddit.message(self.name, subject, text, from_sr)

    async def comments(self, limit=25, **kwargs):
        async for s in self.reddit.get_listing(API_PATH["user_comments"].format(user=self), limit, **kwargs):
            if s["kind"] == self.reddit.comment_kind:
                yield Comment(self.reddit, s["data"])

    async def submissions(self, limit=25, **kwargs):
        async for s in self.reddit.get_listing(API_PATH["user_submissions"].format(user=self), limit, **kwargs):
            if s["kind"] == self.reddit.link_kind:
                yield Submission(self.reddit, s["data"])



import unittest

import apraw


class SubredditTest(unittest.IsolatedAsyncioTestCase):
    def __init__(self, *args, **kwargs):
        super(SubredditTest, self).__init__(*args, **kwargs)

        self._reddit = apraw.Reddit("APB")

    async def asyncSetUp(self):
        self._subreddit = await self._reddit.subreddit("aprawtest")

    async def test_subreddit_id(self):
        print(self._subreddit.id)

    async def test_subreddit_moderators(self):
        moderator_found = False

        async for moderator in self._subreddit.moderators():
            if moderator.name.lower() == "aprawbot":
                moderator_found = True
                break

        self.assertTrue(moderator_found)


if __name__ == "__main__":
    unittest.main()



import asyncio

from .comment import Comment
from .submission import Submission
from .subreddit import Subreddit, ModAction


class ListingGenerator:

    def __init__(self, reddit, endpoint, max_wait=16, kind_filter=[]):
        self.reddit = reddit
        self.endpoint = endpoint
        self.max_wait = max_wait
        self.kind_filter = kind_filter

    async def get(self, limit=25, **kwargs):
        reddit = self.reddit
        last = None

        while True:
            kwargs["limit"] = limit if limit is not None else 100
            if last is not None:
                kwargs["after"] = last
            req = await reddit.get_request(self.endpoint, **kwargs)
            if len(req["data"]["children"]) <= 0:
                break
            for i in req["data"]["children"]:
                if i["kind"] in [reddit.link_kind, reddit.subreddit_kind, reddit.comment_kind]:
                    last = i["data"]["name"]
                elif i["kind"] == reddit.modaction_kind:
                    last = i["data"]["id"]

                if limit is not None: limit -= 1

                if len(self.kind_filter) > 0 and i["kind"] not in self.kind_filter:
                    continue

                if i["kind"] == reddit.link_kind:
                    yield Submission(reddit, i["data"])
                elif i["kind"] == reddit.subreddit_kind:
                    yield Subreddit(self.reddit, i["data"])
                elif i["kind"] == reddit.comment_kind:
                    yield Comment(self.reddit, i["data"])
                elif i["kind"] == reddit.modaction_kind:
                    yield ModAction(self.reddit, i["data"])
            if limit is not None and limit < 1:
                break

    __call__ = get

    async def stream(self, **kwargs):
        wait = 0
        ids = list()

        while True:
            found = False
            async for s in self.get(100, **kwargs):
                if s.id in ids:
                    break
                if len(ids) >= 301:
                    ids = ids[1:]
                ids.append(s.id)
                found = True
                yield s

            if found:
                wait = 1
            else:
                wait *= 2
                if wait > self.max_wait:
                    wait = 1

            await asyncio.sleep(wait)
